# Generated by roxygen2: do not edit by hand

S3method(base::as.data.frame,spicy_categorical_table)
S3method(base::as.data.frame,spicy_continuous_lm_table)
S3method(base::as.data.frame,spicy_continuous_table)
S3method(base::as.data.frame,spicy_regression_table)
S3method(broom::glance,spicy_categorical_table)
S3method(broom::glance,spicy_continuous_lm_table)
S3method(broom::glance,spicy_continuous_table)
S3method(broom::glance,spicy_regression_table)
S3method(broom::tidy,spicy_categorical_table)
S3method(broom::tidy,spicy_continuous_lm_table)
S3method(broom::tidy,spicy_continuous_table)
S3method(broom::tidy,spicy_regression_table)
S3method(knitr::knit_print,spicy_flextable)
S3method(knitr::knit_print,spicy_gt)
S3method(print,spicy_assoc_detail)
S3method(print,spicy_assoc_table)
S3method(print,spicy_categorical_table)
S3method(print,spicy_continuous_lm_table)
S3method(print,spicy_continuous_table)
S3method(print,spicy_cross_table)
S3method(print,spicy_cross_table_list)
S3method(print,spicy_flextable)
S3method(print,spicy_freq_table)
S3method(print,spicy_gt)
S3method(print,spicy_regression_table)
S3method(tibble::as_tibble,spicy_categorical_table)
S3method(tibble::as_tibble,spicy_continuous_lm_table)
S3method(tibble::as_tibble,spicy_continuous_table)
S3method(tibble::as_tibble,spicy_regression_table)
export(as_structured)
export(assoc_measures)
export(build_ascii_table)
export(code_book)
export(contingency_coef)
export(copy_clipboard)
export(count_n)
export(cramer_v)
export(cross_tab)
export(freq)
export(gamma_gk)
export(goodman_kruskal_tau)
export(kendall_tau_b)
export(kendall_tau_c)
export(label_from_names)
export(lambda_gk)
export(mean_n)
export(phi)
export(somers_d)
export(spicy_print_table)
export(sum_n)
export(table_categorical)
export(table_continuous)
export(table_continuous_lm)
export(table_regression)
export(uncertainty_coef)
export(varlist)
export(vl)
export(yule_q)
importFrom(rlang,`%||%`)
importFrom(stats,complete.cases)
importFrom(stats,setNames)
