#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom dplyr pick
#' @importFrom dplyr pull
#' @importFrom dplyr select
#' @importFrom haven labelled
#' @importFrom labelled is.labelled
#' @importFrom labelled to_factor
#' @importFrom labelled var_label
#' @importFrom labelled var_label<-
#' @importFrom rlang enquo
#' @importFrom rlang eval_tidy
#' @importFrom rlang inform
#' @importFrom rlang quo_is_null
#' @importFrom stats complete.cases
#' @importFrom stats na.omit
#' @importFrom stats setNames
#' @importFrom stats xtabs
#' @importFrom stringr str_pad
#' @importFrom tibble as_tibble
#' @importFrom tibble rownames_to_column
#' @importFrom tibble view
#' @importFrom tidyselect all_of
#' @importFrom tidyselect any_of
#' @importFrom tidyselect eval_select
#' @importFrom tidyselect everything
#' @importFrom tidyselect where
#' @importFrom utils capture.output
#' @importFrom utils head
#' @importFrom utils tail
## usethis namespace: end
NULL
