% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_continuous_print.R
\name{print.spicy_continuous_table}
\alias{print.spicy_continuous_table}
\title{Print method for continuous summary tables}
\usage{
\method{print}{spicy_continuous_table}(x, ...)
}
\arguments{
\item{x}{A \code{data.frame} of class \code{"spicy_continuous_table"} as returned
by \code{\link[=table_continuous]{table_continuous()}}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Formats and prints a \code{spicy_continuous_table} object as a styled ASCII
table using \code{\link[=spicy_print_table]{spicy_print_table()}}.
}
\seealso{
\code{\link[=table_continuous]{table_continuous()}}, \code{\link[=spicy_print_table]{spicy_print_table()}}
}
\keyword{internal}
