% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_categorical_print.R
\name{print.spicy_categorical_table}
\alias{print.spicy_categorical_table}
\title{Print method for categorical summary tables}
\usage{
\method{print}{spicy_categorical_table}(x, ...)
}
\arguments{
\item{x}{A \code{data.frame} of class \code{"spicy_categorical_table"} as returned by
\code{\link[=table_categorical]{table_categorical()}} with \code{output = "default"} and \code{styled = TRUE}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Formats and prints a \code{spicy_categorical_table} object as a styled ASCII table using
\code{\link[=spicy_print_table]{spicy_print_table()}}.
}
\seealso{
\code{\link[=table_categorical]{table_categorical()}}, \code{\link[=spicy_print_table]{spicy_print_table()}}
}
\keyword{internal}
