% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross_tab.R
\name{print.spicy_cross_table_list}
\alias{print.spicy_cross_table_list}
\title{Internal print method for lists of cross-tab tables}
\usage{
\method{print}{spicy_cross_table_list}(x, ...)
}
\arguments{
\item{x}{A \code{spicy_cross_table_list} object.}

\item{...}{Additional arguments passed to individual print methods.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints each element of a \code{spicy_cross_table_list} object on its own,
inserting a blank line between tables.
}
\keyword{internal}
